<?php

/**
 * @file
 * Menu callbacks and page-specific functions for the Ooyala module.
 */

const OOYALA_CSS_PATTERN = '([^\r\n,{}]+)(,(?=[^}]*{)|\s*{)'; // validation pattern for CSS rules (JS regex)

/**
 * Menu callback; Display global configuration options for Ooyala fields.
 */
function ooyala_settings_form($form, $form_state) {
  module_load_include('inc', 'ooyala', 'includes/ooyala.api');

  // Display help messages if not yet configured.
  if (empty($form_state['input'])) {
    ooyala_settings_messages();
  }

  $form['ooyala_global_pcode'] = array(
    '#type' => 'textfield',
    '#default_value' => variable_get('ooyala_global_pcode', ''),
    '#title' => t('Ooyala API Key'),
    '#max_length' => 28,
    '#description' => t('Enter your Ooyala API Key.'),
  );

  $form['ooyala_global_secret'] = array(
    '#type' => 'textfield',
    '#default_value' => variable_get('ooyala_global_secret', ''),
    '#title' => t('Ooyala API Secret'),
    '#max_length' => 40,
    '#description' => t('Enter your Ooyala API Secret.'),
  );

  $form['ooyala_player_version'] = array(
    '#title' => t('Ooyala player version'),
    '#type' => 'radios',
    '#options' => array(
      'v4' => 'V4 (Pure JavaScript, HTML5 with Flash fallback, much more mobile friendly)',
      'v3' => 'V3 (HTML5 with Flash fallback, more mobile friendly)',
    ),
    '#required' => TRUE,
    '#default_value' => variable_get('ooyala_player_version', 'v4'),
    '#description' => t('Ooyala introduced a new "V4" player in early 2015. The new player has superior event handling and better responsive design support (100% width players). As of module version <em>7.x-3.x</em>, the V2 player is no longer supported. If you have legacy code, you may be required to use the V2 player and thus an older version of this module, but if unsure, use the V4 player. See <a href="http://help.ooyala.com/video-platform/reference/faq_pbv4.html">the Ooyala V4 FAQ</a> for more information about the V4 player.'),
  );

  // Don't add to the form if no options are available. If this form element is
  // added but the API is unavailable, saving the default value may cause Drupal
  // to report an error when the element validation can't find the option.
  $player_options = ooyala_api_player_list_options();

  if (!empty($player_options)) {
    $selected = variable_get('ooyala_player_id', '');

    if (!isset($player_options[$selected])) {
      list($default) = array_keys($player_options);
    }
    else {
      $default = $selected;
    }

    $form['ooyala_players']['ooyala_player_id'] = array(
      '#type' => 'radios',
      '#title' => t('Ooyala default player (V3 only)'),
      '#required' => TRUE,
      '#default_value' => $default,
      '#options' => $player_options,
      '#description' => t('The default player will be used if no specific player is requested on output. Information can be found in the <a href="http://support.ooyala.com/users/documentation/concepts/chapter_players.html">Ooyala documentation on players.</a>.'),
      '#access' => ooyala_api_available(),
      '#states' => array(
        'visible' => array(
          ':input[name="ooyala_player_version"]' => array('!value' => 'v4'),
        ),
      ),
    );
  }

  $form['ooyala_v3_tweaks'] = array(
    '#title' => t('Tweaks (V3 only)'),
    '#type' => 'radios',
    '#states' => array(
      'visible' => array(
        ':input[name="ooyala_player_version"]' => array('!value' => 'v4'),
      ),
    ),
    '#options' => array(
      'none' => t('None'),
      'html5-force-mp4' => t('Force MP4'),
      'android-enable-hls' => t('Enable HLS (Android)'),
    ),
    '#default_value' => variable_get('ooyala_v3_tweaks', ''),
  );

  $form['ooyala_v3_platform'] = array(
    '#title' => t('Platform (V3 only)'),
    '#type' => 'radios',
    '#states' => array(
      'visible' => array(
        ':input[name="ooyala_player_version"]' => array('!value' => 'v4'),
      ),
    ),
    '#options' => array(
      'html5-priority' => t('HTML5 Priority (Default)'),
      'html5-fallback' => t('HTML5 Fallback'),
      'flash' => t('Flash Playback'),
      'flash-only' => t('Flash-only Playback'),
    ),
    'html5-priority' => array(
      '#description' => t('Sets HTML5 as the preferred playback mechanism but also allows Flash fallback.'),
    ),
    'html5-fallback' => array(
      '#description' => t('Uses Flash but enables fallback to HTML5.'),
    ),
    'flash' => array(
      '#description' => t('For desktop, enables default Adobe Flash playback. For mobile devces, sets the preference to Flash but also allows HTML5 fallback.'),
    ),
    'flash-only' => array(
      '#description' => t('Flash playback only. If Flash is not detected, an error message is displayed and the viewer requested to install the Flash plug-in.')
    ),
    '#default_value' => variable_get('ooyala_v3_platform', 'html5-priority'),
  );

  $form['ooyala_v4_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('V4 Settings'),
    '#states' => array(
      'visible' => array(
        ':input[name="ooyala_player_version"]' => array('value' => 'v4'),
      ),
    ),
    'ooyala_v4_plugins' => array(
      '#prefix' => '<h4>Plugins</h4>',
      '#label' => t('Streaming Plugins'),
      '#description' => t('You must choose at least one streaming plugin.'),
      '#default_value' => variable_get('ooyala_v4_plugins', array('main_html5.min.js')),
      '#type' => 'checkboxes',
      '#required' => TRUE,
      '#options' => array(
        'main_html5.min.js' => t('Default video plugin for HLS and MP4 video streams'),
        'bit_wrapper.min.js' => t('Bitmovin Video Plugin for DASH and HLS'),
        'osmf_flash.min.js' => t('OSMF Flash Video Plugin for HDS'),
      ),
      // Because radio button descriptions are pulled from keys within the
      // checkboxes / radio form element
      'main_html5.min.js' => array('#description' => t('Allows for support of HLS and MP4 video streams, and allows for VAST, VPAID and Freewheel ad playback.')),
      'bit_wrapper.min.js' => array('#description' => t('Allows for support of HLS, DASH and MP4 video streams.')),
      'osmf_flash.min.js' => array('#description' => t('Allows for support of HDS video streams in an Flash player.')),
    ),
    'ooyala_v4_ad_plugins' => array(
      '#label' => t('Ad Plugins'),
      '#description' => t('You may optionally choose one ad plugin.'),
      '#type' => 'radios',
      '#default_value' => variable_get('ooyala_v4_ad_plugins', ''),
      '#options' => array(
        '' => t('None'),
        'freewheel.min.js' => t('Freewheel ad support'),
        'ad_manager_vast.min.js' => t('VAST and VPAID ad support'),
        'google_ima.min.js' => t('Google IMA ad support'),
        'pulse' => t('Pulse ad network support'),
      ),
    ),
    'ooyala_v4_pulse_options' => array(
      '#name' => 'ooyala_v4_pulse_options',
      '#type' => 'hidden',
      '#title' => t('Additional options for the Pulse plugin'),
      '#theme' => 'ooyala_json_textarea',
      '#default_value' => variable_get('ooyala_v4_pulse_options', array()),
      '#rows' => 8,
      '#description' => t('Review the <a href="https://www.dropbox.com/s/70u5h4oen6jlnbo/Ooyala-HTML5playerV4-Pulseintegration_20160406.pdf?dl=0">Pulse integration reference</a> for details on acceptable parameters.'),
      '#states' => array(
        'visible' => array(
          ':input[name="ooyala_v4_ad_plugins"]' => array('value' => 'pulse'),
        ),
      ),
    ),
    'ooyala_v4_pulse_plugin' => array(
      '#type' => 'textfield',
      '#title' => t('Override Pulse plugin URL:'),
      '#attributes' => array(
        'placeholder' => OOYALA_V4_DEFAULT_PULSE_PLUGIN,
      ),
      '#default_value' => variable_get('ooyala_v4_pulse_plugin'),
      '#states' => array(
        'visible' => array(
          ':input[name="ooyala_v4_ad_plugins"]' => array('value' => 'pulse'),
        ),
      ),
    ),
    'ooyala_v4_optional_plugins' => array(
      '#title' => t('Optional Plugins'),
      '#description' => t('Plugins that provide additional functionality to your player.'),
      '#type' => 'checkboxes',
      '#options' => array(
        'discovery_api.min.js' => t('Discovery'),
      ),
      '#default_value' => array_filter(variable_get('ooyala_v4_optional_plugins', array())),
      'discovery_api.min.js' => array('#description' => t('Keep users engaged by directing them to related and relevant content on your site.')),
    ),
    'ooyala_v4_skin_json' => array(
      '#name' => 'ooyala_v4_skin_json',
      '#type' => 'hidden',
      '#title' => t('Additional JSON Skin'),
      '#default_value' => variable_get('ooyala_v4_skin_json', ''),
      '#rows' => 8,
      '#theme' => 'ooyala_json_textarea',
      '#description' => t('Review the <a href="http://support.ooyala.com/developers/documentation/reference/pbv4_skin_schema_docs.html">JSON skinning reference</a> for details on acceptable parameters.'),
    ),
    'ooyala_v4_custom_css' => array(
      '#title' => t('Custom CSS Skin'),
      '#type' => 'textarea',
      '#default_value' => variable_get('ooyala_v4_custom_css'),
      '#placeholder' => t('Additional CSS rules to be applied to all players'),
      '#rows' => 8,
      '#description' => t('Review the <a href="http://support.ooyala.com/developers/documentation/concepts/pbv4_css.html">CSS skinning reference</a> for details on how to style your player using CSS.'),
    ),
  );
  $form['#validate'][] = 'ooyala_settings_v4_validate';

  $width = variable_get('ooyala_video_width');
  $height = variable_get('ooyala_video_height');
  $form['ooyala_dimensions'] = array(
    '#title' => t('Player dimensions'),
    '#type' => 'radios',
    '#options' => array(
      'auto' => t('Automatic'),
      'fixed' => t('Fixed size'),
    ),
    '#default_value' => (empty($width) && empty($height)) ? 'auto' : 'fixed',
  );

  $form['ooyala_video_width'] = array(
    '#title' => t('Player width'),
    '#type' => 'textfield', 
    '#description' => t('The width of the video player when viewing content.'),
    '#default_value' => $width,
    '#size' => 6,
    '#field_suffix' => ' ' . t('pixels'),
    '#states' => array(
      'visible' => array(
        '[name="ooyala_dimensions"]' => array('value' => 'fixed'),
      )
    ),
  );

  $form['ooyala_video_height'] = array(
    '#title' => t('Player height'),
    '#type' => 'textfield', 
    '#description' => t('The height of the video player when viewing content.'),
    '#default_value' => $height,
    '#size' => 6,
    '#field_suffix' => ' ' . t('pixels'),
    '#states' => array(
      'visible' => array(
        '[name="ooyala_dimensions"]' => array('value' => 'fixed'),
      )
    ),
  );

  // Allow the user to determine what type of node this should be.
  $options = ooyala_type_names();
  $default = variable_get('ooyala_primary_content_type', 'video');
  $form['ooyala_primary_content_type'] = array(
    '#type' => 'select',
    '#description' => t('This content type will be used as the default for the <a href="!url">batch Backlot video import</a>.', array('!url' => url('admin/content/ooyala/import'))),
    '#title' => t('Default content type'),
    '#options' => $options,
    '#default_value' => isset($options[$default]) ? $default : NULL,
  );

  if (empty($options)) {
    $form['ooyala_primary_content_type']['#disabled'] = TRUE;
    $form['ooyala_primary_content_type']['#options'][''] = t('No available types');
    $form['ooyala_primary_content_type']['#description'] = t('This content type will be used as the default for the <a href="!url">batch Backlot video import</a>.', array('!url' => url('admin/content/ooyala/import'))) . ' ' . t('There are currently no content types that contain an Ooyala field. Add an Ooyala field under <em>Manage fields</em> from the <a href="!url">Content type configuration</a>.', array('!url' => url('admin/structure/types')));
  }

  if (module_exists('colorbox')) {
    $form['ooyala_colorbox_height'] = array(
      '#type' => 'textfield', 
      '#description' => t('The height that the colorbox window will have when showing full player video.'),
      '#title' => t('Colorbox video height'),
      '#default_value' => variable_get('ooyala_colorbox_height', 400),
    );

    $form['ooyala_colorbox_width'] = array(
      '#type' => 'textfield', 
      '#description' => t('The width that the colorbox window will have when showing full player video.'),
      '#title' => t('Colorbox video width'),
      '#default_value' => variable_get('ooyala_colorbox_width', 500),
    );

    $form['ooyala_colorbox_autoplay'] = array(
      '#type' => 'checkbox',
      '#description' => t('When the video is shown in a colorbox dialog you usually want it to play automatically, uncheck this if you don\'t want this behavior.'),
      '#title' => t('Autoplay video in Colorbox'),
      '#default_value' => TRUE,
    );

  }

  $form['ooyala_reporting_level'] = array(
    '#type' => 'radios',
    '#title' => t('Reporting level'),
    '#options' => array(
      '1' => t('Verbose - display errors and status messages'),
      '0' => t('Quiet - log errors to watchdog (recommended for production sites)'),
    ),
    '#default_value' => variable_get('ooyala_reporting_level', 1),
  );

  $ping_help = '';
  $ping_help .= t('The Ooyala Ping is an option that you can use to retrieve thumbnails and publish content after it has completed processing by the Ooyala servers.');
  $ping_help .= ' ' . t('To enable the Ooyala ping, you must set up the "API Ping URL" <strong>@url</strong> in your <a href="http://www.ooyala.com/backlot/web">Ooyala Backlot account</a> (under the "Account" => "Developers" tab).', array('@url' => url('ooyala/ping', array('absolute' => TRUE))));

  $form['ooyala_ping'] = array(
    '#type' => 'fieldset',
    '#title' => t('Ooyala ping settings'),
    '#collapsible' => FALSE,
    '#description' => $ping_help,
  );

  $form['ooyala_ping']['ooyala_autopublish'] = array(
    '#type' => 'checkbox',
    '#title' => t('Auto-publish content when videos finish processing.'),
    '#default_value' => variable_get('ooyala_autopublish', FALSE),
    '#description' => t('Upon receiving a ping, publish nodes that contain the video that has finished processing.'),
  );

  $form['ooyala_ping']['ooyala_ping_reporting_verbose'] = array(
    '#type' => 'checkbox',
    '#title' => t('Verbose ping reporting'),
    '#default_value' => t(variable_get('ooyala_ping_reporting_verbose', FALSE)),
    '#description' => t('Log additional messages to the watchdog during each step processing an Ooyala ping including a complete copy of the incoming ping request.'),
  );

  // Add the save button and normal submit handlers.
  $form = system_settings_form($form);
  array_unshift($form['#submit'], 'ooyala_settings_submit');
  $form['buttons']['#weight'] = 20;

  drupal_add_js(drupal_get_path('module', 'ooyala') . '/ooyala_settings.js');
  drupal_add_css('.ooyala-invalid-value { outline: 1px solid red; }', 'inline');

  $form['#attached']['js'][] = array(
    'type' => 'setting',
    'data' => array(
      'Ooyala' => array(
        'cssPattern' => OOYALA_CSS_PATTERN,
      ),
    ),
  );

  return $form;
}

/**
 * Extra submit handler for ooyala_settings().
 */
function ooyala_settings_submit($form, &$form_state) {
  if ($form_state['values']['ooyala_dimensions'] === 'auto') {
    $form_state['values']['ooyala_video_width'] = NULL;
    $form_state['values']['ooyala_video_height'] = NULL;
  }
  else {
    $form_state['values']['ooyala_video_width'] = (int) $form_state['values']['ooyala_video_width'];
    $form_state['values']['ooyala_video_height'] = (int) $form_state['values']['ooyala_video_height'];
  }

  // Set values from custom JSON input elements
  variable_set('ooyala_v4_pulse_options', $form_state['values']['ooyala_v4_pulse_options']);
  variable_set('ooyala_v4_skin_json', $form_state['values']['ooyala_v4_skin_json']);

  // Unset this value entirely to prevent a variable from being set.
  unset($form_state['values']['ooyala_dimensions']);
}

/**
 * Print out messages checking the current Ooyala configuration.
 */
function ooyala_settings_messages() {
  if (!variable_get('ooyala_global_pcode', '')) {
    $help = '';
    $help .= t('Welcome to the Ooyala module! Before you can upload any content you need to configure your site for syncronizing with Ooyala. To do this, you need to both enter and retrieve some information to and from Ooyala.');
    $steps = array(
      t('If you haven\'t yet, you need to <a target="_blank" href="http://www.ooyala.com/free_trial">create an Ooyala account</a>. After you have an account, you need to sign-in to the <a target="_blank" href="https://backlot.ooyala.com/backlot/web">Ooyala admin interface (Backlot)</a>.'),
      t('Once logged in to Backlot, click on the "Account" tab. Then click on the "Developers" tab. Enter the following into "API Ping URL":') . '<br /><strong>' . url('ooyala/ping', array('absolute' => TRUE)) . '</strong><br />' . t('Note that your site must be publicly accessible from the internet for this step to work. If your site is not publicly available, some features of the Ooyala module will not work (auto-publishing), while other features will work but be delayed until cron jobs (thumbnail and attribute retreival).'),
      t('On that same page, there are two values for "API Key" And "API Secret". Copy and paste these values from Backlot into the fields below.'),
      t('The Ooyala module is very dependent upon cron jobs, ensure that "Cron maintenance tasks" is set up properly on your site by checking your <a href="!url">site status report</a>.', array('!url' => url('admin/reports/status'))),
    );
    $help .= theme('item_list', array('items' => $steps, 'title' => NULL, 'type' => 'ol'));
    drupal_set_message($help);
  }
  elseif (ooyala_api_available()) {
    $details = array();
    ooyala_api_video_query(array('limit' => 1), $details);
    if ($details) {
      drupal_set_message(t('The Ooyala servers have been contacted and your API Key and Secret code have been confirmed.'), 'status', FALSE);
    }
    else {
      drupal_set_message(t('Check that your API Key and Secret code values are correct. These can be obtained from your <a href="https://backlot.ooyala.com/backlot/web">Ooyala Backlot account</a> under the "Account" => "Developers" sub-tab.'), 'error');
    }
  }
  else {
    drupal_set_message(t('The Ooyala service could not be reached. Check that your server is able to make HTTP requests.'), 'warning');
  }
}

/**
 * Validate that at least one streaming plugin has been selected for V4 configuration
 */
function ooyala_settings_v4_validate($form, &$form_state) {
  if ($form_state['values']['ooyala_player_version'] === 'v4') {
    $selected = array_filter($form_state['values']['ooyala_v4_plugins']);
    if (empty($selected)) {
      form_set_error('ooyala_v4_settings][ooyala_v4_plugins', t("You must select at least one streaming plugin"));
    }
  }
}

/**
 * Element #validate function to ensure the entered directory is writable.
 */
function ooyala_settings_path_validate($element) {
  if (!empty($element['#value'])) {
    $uri = file_build_uri($element['#value']);
    $path = file_prepare_directory($uri, FILE_CREATE_DIRECTORY);
    if (!$path) {
      form_error($element, t("The directory %path is not writable or could not be created. Check your server's file system permissions.", array('%path' => $path)));
    }
  }
  return $element;
}

/**
 * Element #validate function to ensure a file exists.
 */
function ooyala_settings_file_validate($element) {
  $uri = file_build_uri($element['#value']);
  $path = file_prepare_directory($uri, FILE_CREATE_DIRECTORY);
  if (!empty($element['#value']) && !is_file($path)) {
    form_error($element, t('The image at %path could not be found.', array('%path' => $path)));
  }
  return $element;
}

/**
 * Menu callback; Retrieve a new thumbnail from the Ooyala servers.
 */
function ooyala_refresh_thumbnail() {
  module_load_include('inc', 'ooyala', 'includes/ooyala.api');
  $token = drupal_get_token();

  $data = array();
  if (TRUE || isset($_GET['token']) && $_GET['token'] == $token) {
    if (isset($_GET['embed_code']) && $_GET['embed_code']) {
      $embed_code = $_GET['embed_code'];
      // Try to retrieve a new thumbnail.
      $thumb_path = ooyala_api_fetch_image($embed_code);
      if ($thumb_path != FALSE) {
        $data['content'] = theme('ooyala_upload_preview', array('embed_code' => $embed_code));
        $data['message'] = '';
        $data['error'] = 0;

        // Flush out all existing image styles.
        $styles = image_styles();
        foreach ($styles as $style) {
          image_style_flush($style);
        }
      }
      else {
        $data['content'] = theme('ooyala_upload_preview');
        $data['message'] = t('A thumbnail was not able to be retrieved. Check that the video code is correct and that the video has finished processing.');
        $data['error'] = 0;
      }

      $data['content'] .= theme('ooyala_video_data', array('data_type' => 'length', 'data' => ooyala_api_video_property($embed_code, 'length')));
      $data['content'] .= theme('ooyala_video_data', array('data_type' => 'status', 'data' =>  ooyala_api_video_property($embed_code, 'status')));

    }
    else {
      $data['message'] = t('A video code must be provided before it can be previewed.');
      $data['error'] = 1;
    }
  }
  else {
    $data['message'] = t('Invalid token.');
    $data['error'] = 1;
  }

  $data['field_id'] = isset($_GET['field_id']) ? $_GET['field_id'] : '';

  drupal_json_output($data);
}

